# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os, subprocess
import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

import re

project = 'Shimatta Reflow Controller'
copyright = '2020, Mario Hüttel'
author = 'Mario Hüttel'

# The full version, including alpha/beta/rc tags.
release = re.sub('^v', '', os.popen('git describe --always --tags --dirty').read().strip())
# The short X.Y version.
version = release

try:
	os.mkdir('../build/_doxygen')
except FileExistsError:
	pass
subprocess.call('doxygen Doxyfile.in', shell=True)

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
	'sphinx_rtd_theme',
	'sphinx.ext.autodoc',
	'sphinx.ext.todo',
	'sphinxcontrib.blockdiag',
	'breathe'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

breathe_projects = {
	"Reflow Controller Firmware": "../build/_doxygen/xml/"
}

breathe_domain_by_extension = { "h" : "c",
				"c" : "c" }
breathe_default_project = "Reflow Controller Firmware"
breathe_default_members = ('members', 'undoc-members')

blockdiag_html_image_format = 'SVG'
breathe_show_define_initializer = True
blockdiag_latex_image_format = 'PDF'

todo_include_todos = True
