/* Reflow Oven Controller
*
* Copyright (C) 2020  Mario Hüttel <mario.huettel@gmx.net>
*
* This file is part of the Reflow Oven Controller Project.
*
* The reflow oven controller is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* The Reflow Oven Control Firmware is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the reflow oven controller project.
* If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __WATCHDOG_H__
#define __WATCHDOG_H__

#include <reflow-controller/safety/safety-config.h>
#include <stdint.h>
#include <stdbool.h>

/**
 * @brief Setup the watchdog for the safety controller
 * @param Prescaler to use for the 32 KHz LSI clock
 * @return 0 if successful
 * @note Once the watchdog is enabled, it cannot be turned off!
 */
int watchdog_setup(uint8_t prescaler);

/**
 * @brief Reset watchdog counter
 * @param magic Magic value to prevent this fuinction from being called randomly
 * @return 0 if successful
 */
int watchdog_ack(uint32_t magic);

/**
 * @brief Check if reset was generated by the watchdog.
 * @note This also clears the relevant flag, so the function will return false when called a second time
 * @return true, if reset was generated by the watchdog
 */
bool watchdog_check_reset_source(void);

#endif /* __WATCHDOG_H__ */
