/* Reflow Oven Controller
 *
 * Copyright (C) 2020  Mario Hüttel <mario.huettel@gmx.net>
 *
 * This file is part of the Reflow Oven Controller Project.
 *
 * The reflow oven controller is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The Reflow Oven Control Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the reflow oven controller project.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdint.h>
#include <stdbool.h>

/**
 * @brief Init the backup ram and make it accesible
 * @param use_backup_regulator Enable the Backup VBAT regulator. It will be used, when VDD is powered off
 */
void backup_ram_init(bool use_backup_regulator);

/**
 * @brief Disable access to the backup RAM. This saves power
 */
void backup_ram_disable(void);

/**
 * @brief Whis function overwrites the backup RAM with 0x00000000
 */
void backup_ram_wipe(void);

/**
 * @brief Read data from the backup RAM
 * @param addr Address offset inside memory
 * @param data Read data
 * @param count amount of 32 bit words to read
 * @return 0 if successful
 */
int backup_ram_get_data(uint32_t addr, uint32_t *data, uint32_t count);

/**
 * @brief Write data structure to backup RAM
 * @param[in] data Data to write.
 * @param count Count of 32 bit words to write
 * @return 0 if successful
 */
int backup_ram_write_data(uint32_t addr, const uint32_t *data, uint32_t count);

uint32_t backup_ram_get_size_in_words(void);

volatile void *backup_ram_get_base_ptr(void);
